# Generated by Django 4.2.17 on 2024-12-07 14:11

import baykeshop.db.fields
from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeArticleCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='article.baykearticlecategory', verbose_name='父级')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '文章分类',
                'verbose_name_plural': '文章分类',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeSidebar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='名称')),
                ('icon', models.CharField(blank=True, default='mdi mdi-menu', max_length=50, verbose_name='图标')),
                ('module', models.CharField(choices=[('search', '搜索'), ('archive', '归档'), ('nav', '导航栏'), ('tags', '标签'), ('custom', '自定义')], default='custom', max_length=255, verbose_name='模块')),
                ('content', models.TextField(blank=True, default='', help_text='自定义内容，仅在自定义模块下生效，支持html语法', verbose_name='自定义内容')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '侧边栏',
                'verbose_name_plural': '侧边栏',
                'ordering': ['order'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeArticleTags',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='标签名称')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '文章标签',
                'verbose_name_plural': '文章标签',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='BaykeArticleContent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('title', models.CharField(max_length=50, verbose_name='标题')),
                ('content', baykeshop.db.fields.RichTextField(verbose_name='内容')),
                ('image', models.ImageField(blank=True, null=True, upload_to='article', verbose_name='图片')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='article.baykearticlecategory', verbose_name='文章分类')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
                ('tags', models.ManyToManyField(blank=True, to='article.baykearticletags', verbose_name='文章标签')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
            ],
            options={
                'verbose_name': '文章内容',
                'verbose_name_plural': '文章内容',
                'ordering': ['-created_time'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from django import forms
from django.contrib.auth.forms import (
    UserCreationForm, AuthenticationForm, PasswordChangeForm
)
from django.contrib.auth import password_validation
from django.utils.translation import gettext_lazy as _

from baykeshop.forms.mixins import BaseFormMixins
from baykeshop.forms import widgets
from baykeshop.contrib.shop.models import BaykeShopOrdersComment
from .models import BaykeShopUserAddress, BaykeShopUser


class LoginForm(BaseFormMixins, AuthenticationForm):
    """ 登录表单 """
    username = forms.CharField(
        label=_('用户名'),
        widget=widgets.TextInput(
            attrs={'placeholder': _('请输入用户名'), 'autofocus': True},
            icon_position='bk-has-icons-left bk-has-icons-right',
            icons_class={'left': 'mdi mdi-account', 'right': 'mdi mdi-check'}
        ),
        required=True,
    )
    password = forms.CharField(
        label=_('密码'),
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请输入密码')},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        ),
        required=True,
    )


class RegisterForm(BaseFormMixins, UserCreationForm):
    """ 注册表单 """

    username = forms.CharField(
        label=_("用户名"),
        strip=False,
        widget=widgets.TextInput(
            attrs={'placeholder': _('请输入用户名'), "autofocus": True},
            icon_position='bk-has-icons-left bk-has-icons-right',
            icons_class={'left': 'mdi mdi-account', 'right': 'mdi mdi-check'}
        )
    )
    password1 = forms.CharField(
        label=_("Password"),
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请输入密码'), "autocomplete": "new-password"},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        ),
        strip=False,
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请再次输入密码')},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        ),
        help_text=_("Enter the same password as before, for verification."),
    )


class ChangePasswordForm(BaseFormMixins, PasswordChangeForm):
    """ 修改密码表单 """

    old_password = forms.CharField(
        label=_("Old password"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请输入旧密码'), "autocomplete": "current-password"},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        )
    )
    new_password1 = forms.CharField(
        label=_("New password"),
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请输入新密码'), "autocomplete": "new-password"},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        ),
        strip=False,
        help_text=password_validation.password_validators_help_text_html(),
    )
    new_password2 = forms.CharField(
        label=_("New password confirmation"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={'placeholder': _('请再次输入新密码'), "autocomplete": "new-password"},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-lock', 'right': ''}
        ),
        help_text=_("Enter the same password as before, for verification."),
    )
        

class BaykeShopUserAddressForm(BaseFormMixins, forms.ModelForm):
    """ 收货地址表单 """
    
    class Meta:
        model = BaykeShopUserAddress
        fields = ['name', 'phone', 'province', 'city', 'district', 'address', 'is_default']
        widgets = {
            'name': widgets.TextInput(
                attrs={'placeholder': _('请输入收货人')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-account', 'right': ''}
            ),
            'phone': widgets.TextInput(
                attrs={'placeholder': _('请输入手机')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-phone', 'right': ''}
            ),
            'province': widgets.TextInput(
                attrs={'placeholder': _('请输入省')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'city': widgets.TextInput(
                attrs={'placeholder': _('请输入市')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'district': widgets.TextInput(
                attrs={'placeholder': _('请输入区')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'address': widgets.TextInput(
                attrs={'placeholder': _('请输入详细地址')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            )
        }


class BaykeShopUserProfileForm(BaseFormMixins, forms.ModelForm):
    """ 个人资料表单 """
    
    email = forms.EmailField(
        label=_("邮箱"),
        widget=widgets.TextInput(
            attrs={'placeholder': _('请输入邮箱'), "autocomplete": "email", 'type': 'email'},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-email', 'right': ''}
        )
    )

    class Meta:
        model = BaykeShopUser
        fields = ['avatar', 'gender', 'nickname', 'birthday', 'email', 'mobile', 'qq', 'wechat', 'description']
        widgets = {
            'nickname': widgets.TextInput(
                attrs={'placeholder': _('请输入昵称')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-account', 'right': ''},
            ),
            'mobile': widgets.TextInput(
                attrs={'placeholder': _('请输入手机号码')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-phone', 'right': ''}
            ),
            'gender': widgets.Select(
                attrs={'placeholder': _('请选择性别')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-gender-male-female', 'right': ''}
            ),
            'birthday': forms.DateInput(attrs={'type': 'date', 'class': 'bk-input' }, format='%Y-%m-%d'),
            'qq': widgets.TextInput(
                attrs={'placeholder': _('请输入QQ')},
               icon_position='bk-has-icons-left',
               icons_class={'left': 'mdi mdi-qqchat', 'right': ''}
            ),
            'wechat': widgets.TextInput(
                attrs={'placeholder': _('请输入微信')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-wechat', 'right': ''}
            ),
            'description': forms.Textarea(
                attrs={'placeholder': _('请输入简介'), 'class': 'bk-textarea bk-has-fixed-size'},
            )
        }


class BaykeShopOrdersCommentForm(BaseFormMixins, forms.ModelForm):
    """订单评论表单"""

    class Meta:
        model = BaykeShopOrdersComment
        fields = ('content', 'score',)
        widgets = {
            'content': forms.Textarea(
                attrs={'placeholder': _('请输入评论内容'), 'class': 'bk-textarea', 'rows': 5, 'cols': 50},
            ),
            'score': widgets.Select(
                attrs={'placeholder': _('请选择评分')},
                icon_position='bk-has-icons-left',
                icons_class={'left':'mdi mdi-star', 'right': ''}
            )
        }


import json
from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.utils.html import format_html
from django.utils import timezone
from django.template.loader import render_to_string
# Register your models here.
from baykeshop.sites import admin as bayke_admin
from .forms import BaykeShopGoodsSKUForm
from .models import *


class BaykeShopCategoryInline(bayke_admin.TabularInline):
    model = BaykeShopCategory
    extra = 1

@admin.register(BaykeShopCategory)
class BaykeShopCategoryAdmin(bayke_admin.ModelAdmin):
    list_display = ['name', 'parent', 'order', 'is_floor', 'is_nav', 'is_show']
    list_editable = ['order', 'is_show']
    list_filter = ['is_show',]
    search_fields = ['name']
    readonly_fields = ['parent']
    inlines = [BaykeShopCategoryInline]

    fieldsets = (
        (None, {
            'fields': (
                'name', 'icon', 'order',  'is_floor', 'is_nav', 'is_show', 
            )
        }),
    )

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'parent':
            kwargs['queryset'] = BaykeShopCategory.objects.filter(parent__isnull=False)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_inline_instances(self, request, obj=None):
        if obj and obj.parent:
            return []
        return super().get_inline_instances(request, obj)


class BaykeShopGoodsSKUInline(bayke_admin.StackedInline):
    model = BaykeShopGoodsSKU
    extra = 0
    form = BaykeShopGoodsSKUForm
    readonly_fields = ('sales',)


class BaykeShopGoodsImagesInline(bayke_admin.TabularInline):
    model = BaykeShopGoodsImages
    extra = 0
    

@admin.register(BaykeShopGoods)
class BaykeShopGoodsAdmin(bayke_admin.ModelAdmin):
    list_display = ('id', 'name', 'image', 'brand', 'price', 'sales', 'stock', 'is_recommend', 'created_time')
    list_display_links = ('id', 'name', 'image')
    list_editable = ('is_recommend',)
    list_filter = ('category', 'brand')
    search_fields = ('name', 'category__name', 'brand__name')
    inlines = [BaykeShopGoodsSKUInline, BaykeShopGoodsImagesInline]
    fieldsets = (
        (_('基本信息'), {
            'fields': ('name', 'category', 'brand',)
        }),
        (_('商品详情'), {
            # 'classes': ('collapse',),
            'fields': ('keywords', 'description', 'detail',)
        }),
    )
    filter_horizontal = ("category", )

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'category':
            kwargs['queryset'] = BaykeShopCategory.objects.filter(parent__isnull=False)
        return super().formfield_for_manytomany(db_field, request, **kwargs)
    
    @admin.display(description='商品价格')
    def price(self, obj):
        return round(obj.price, 2)
    
    @admin.display(description='商品销量')
    def sales(self, obj):
        return obj.sales
    
    @admin.display(description='商品库存')
    def stock(self, obj):
        return obj.stock
    
    @admin.display(description='商品图片')
    def image(self, obj):
        return format_html(
            '<img src="/media/{}" width="64" height="64" />',
            obj.image_url
        )


@admin.register(BaykeShopBrand)
class BaykeShopBrandAdmin(bayke_admin.ModelAdmin):
    '''Admin View for BaykeShopBrand'''
    list_display = ('id', 'name', 'image', 'order', 'created_time')
    list_display_links = ('id', 'name')
    search_fields = ('name', 'description')
    list_editable = ('order',)

    fieldsets = (
        (_('基本信息'), {
            'fields': ('image', 'name', 'description', 'order')
        }),
    )


class BaykeShopOrdersGoodsInline(bayke_admin.TabularInline):
    model = BaykeShopOrdersGoods
    extra = 0
    exclude = ('specs', 'sku', 'detail', 'image')
    readonly_fields = (
        '_image', 'name', 'price', 'quantity', '_specs', 'sku_sn' 
    )

    @admin.display(description='规格')
    def _specs(self, obj):
        specs = obj.specs
        if isinstance(specs, str):
            specs = json.loads(obj.specs)
        if not specs: return '-'
        return ', '.join([f"{spec['parent__name']}:{spec['name']}" for spec in specs])
    
    @admin.display(description='商品图片')
    def _image(self, obj):
        if not obj.image: return '-'
        return format_html(
            '<img src="/media/{}" width="64" height="64" />',
            obj.image
        )

    def has_delete_permission(self, request, obj=None):
        return False
    
    def has_add_permission(self, request, obj=None):
        return False


@admin.register(BaykeShopOrders)
class BaykeShopOrdersAdmin(bayke_admin.ModelAdmin):
    list_display = (
        'id', 'user', 'order_sn', 'order_skus', 'status', 'pay_type', 
        'pay_price', 'is_verify', 'is_comment', 'created_time', 'pay_time'
    )
    list_display_links = ('id', 'user', 'order_sn')
    search_fields = ('id', 'user__username', 'user__nickname')
    list_filter = ('status', 'pay_type', 'is_verify', 'is_comment')
    readonly_fields = (
        'order_sn', 'user', 'pay_type', 'is_comment',
        'pay_sn', 'pay_time', 'is_verify', 'verify_time'
    )
    inlines = [
        BaykeShopOrdersGoodsInline,
    ]
    actions = ['shipments', 'verify']
   
    def has_add_permission(self, request):
        return False
    
    def has_change_permission(self, request, obj=None):
        # 未支付和未发货订单可操作修改
        if obj and obj.status in [0, 1]:
            return super().has_change_permission(request, obj)
        return False
    
    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj)
        # 已支付订单不能再修改支付金额
        if obj and obj.status >= BaykeShopOrders.OrderStatus.SHIPPED:
            readonly_fields = list(readonly_fields) + ['pay_price',]
        return readonly_fields

    @admin.display(description='订单商品')
    def order_skus(self, obj):
        queryset = obj.baykeshopordersgoods_set.all()
        return render_to_string('baykeshop/admin/ordersgoods.html', {'queryset': queryset})

    @admin.action(description='所选订单 发货')
    def shipments(self, request, queryset):
        for item in queryset:
            if item.status != BaykeShopOrders.OrderStatus.PAID: 
                continue
            item.status = BaykeShopOrders.OrderStatus.SHIPPED
            item.save()
        self.message_user(request, '发货成功')
    
    @admin.action(description='所选订单 核销')
    def verify(self, request, queryset):
        for item in queryset:
            if item.status != BaykeShopOrders.OrderStatus.VERIFY: 
                continue
            item.status = BaykeShopOrders.OrderStatus.SIGNED
            item.is_verify = True
            item.pay_time = timezone.now()
            item.save()
        self.message_user(request, '核销成功')

# 规格值
class BaykeShopSpecInline(bayke_admin.TabularInline):
    model = BaykeShopSpec
    extra = 1
    verbose_name = _('规格值')
    verbose_name_plural = _('规格值')


@admin.register(BaykeShopSpec)
class BaykeShopSpecAdmin(bayke_admin.ModelAdmin):
    '''Admin View for BaykeShopSpec'''
    list_display = ('id', 'name', 'parent', 'order', 'is_show', 'created_time')
    list_display_links = ('id', 'name')
    search_fields = ('name',)

    fieldsets = (
        (_('规格名称'), {
            'fields': ('name', 'order', 'is_show')
        }),
    )
    readonly_fields = ('parent',)
    inlines = [BaykeShopSpecInline]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'parent':
            kwargs['queryset'] = BaykeShopSpec.objects.filter(parent__isnull=False)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_inline_instances(self, request, obj=None):
        if obj and obj.parent:
            return []
        return super().get_inline_instances(request, obj)


@admin.register(BaykeShopOrdersComment)
class BaykeShopOrdersCommentAdmin(bayke_admin.ModelAdmin):
    '''Admin View for BaykeShopOrdersComment'''
    list_display = ('id', 'user', 'order', 'score', 'content', 'reply_user', 'status', 'created_time')
    list_display_links = ('id', 'user', 'order')
    list_editable = ('status',)
    search_fields = ('user__username', 'user__nickname')
    list_filter = ('score',)
    readonly_fields = ('user', 'order', 'content', 'reply_user', 'score')

    fieldsets = (
        (_('评论信息'), {
            'fields': ('user', 'order', 'content', 'score')
        }),
        (_('回复信息'), {
            'fields': ('reply_user', 'reply_content', 'status')
        }),
    )

    def save_model(self, request, obj, form, change):
        obj.reply_user = request.user
        return super().save_model(request, obj, form, change)
    

    def has_add_permission(self, request, obj=None):
        return False


from django.db import models
from django.utils.translation import gettext_lazy as _

from baykeshop.db import BaseOrdersModel, BaseOrdersGoodsModel
from .goods import BaykeShopGoodsSKU


class BaykeShopOrders(BaseOrdersModel):
    """订单表"""

    class Meta(BaseOrdersModel.Meta):
        verbose_name = _('订单')
        verbose_name_plural = verbose_name
        ordering = ['-created_time']
        
    def __str__(self):
        return self.order_sn
    
    @property
    def total_price(self):
        return self.pay_price
    
    @property
    def total_quantity(self):
        return self.baykeshopordersgoods_set.count()
    

class BaykeShopOrdersGoods(BaseOrdersGoodsModel):
    """订单商品表"""
    sku = models.ForeignKey(BaykeShopGoodsSKU, on_delete=models.SET_NULL, verbose_name=_('商品'), null=True)
    orders = models.ForeignKey(BaykeShopOrders, on_delete=models.CASCADE, verbose_name=_('订单'))

    class Meta:
        verbose_name = _('订单商品')
        verbose_name_plural = verbose_name
        ordering = ['-created_time']
        constraints = [
            models.UniqueConstraint(fields=['sku', 'orders'], name='sku_orders_unique')
        ]

    def __str__(self):
        return self.name


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from .base import BaseModel
from .fields import RichTextField

User = get_user_model()


class BaseArticleModel(BaseModel):
    """ 文章基类模型 """
    title = models.CharField(max_length=50, verbose_name=_('标题'))
    # content = models.TextField(verbose_name=_('内容'))
    content = RichTextField(verbose_name=_('内容'))
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('作者'), blank=True, null=True)
    image = models.ImageField(upload_to='article', blank=True, null=True, verbose_name=_('图片'))
    order = models.IntegerField(default=0, verbose_name=_('排序'))

    class Meta:
        ordering = ['-order', '-created_time']
        abstract = True



from django.template import Library
from baykeshop.contrib.system.models import Visit, BaykeBanners, BaykeDictModel

register = Library()

@register.simple_tag
def dict_value(key):
    """获取字典值"""
    return BaykeDictModel.get_key_value(key)

@register.simple_tag
def visit_count(content_object:Visit, key:str):
    """访问统计"""
    return Visit.objects.get_uv_pv_count(content_object).get(key)


@register.inclusion_tag("baykeshop/tags/banners.html")
def banners_template():
    """ 轮播图 """
    return {
        'banners': BaykeBanners.objects.filter(is_show=True)
    }


from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.views import PasswordChangeView
from django.contrib.auth.views import RedirectURLMixin
from django.contrib.messages.views import SuccessMessageMixin
from django.views.generic import TemplateView, FormView, UpdateView, ListView, CreateView, DeleteView

from django.utils.translation import gettext_lazy as _
from django.urls import reverse_lazy

from baykeshop.contrib.member.models import BaykeShopUserAddress, BaykeShopUser
from baykeshop.contrib.member.forms import ChangePasswordForm, BaykeShopUserAddressForm, BaykeShopUserProfileForm

class BaykeShopUserProfileView(LoginRequiredMixin, TemplateView):
    """ 个人中心 """
    template_name = 'baykeshop/member/profile.html'
    extra_context = {'title': _('个人中心'),}
    

class BaykeShopUserPasswordView(LoginRequiredMixin, SuccessMessageMixin, PasswordChangeView):
    """ 修改密码 """
    template_name = 'baykeshop/member/password.html'
    extra_context = {'title': _('修改密码'),}
    success_url = reverse_lazy('member:profile')
    login_url = reverse_lazy('member:login')
    form_class = ChangePasswordForm
    success_message = _('密码修改成功')


class BaykeShopUserAddressListView(LoginRequiredMixin, ListView):
    """ 收货地址列表 """
    template_name = 'baykeshop/member/address_list.html'
    model = BaykeShopUserAddress
    context_object_name = 'address_list'
    extra_context = {'title': _('收货地址列表'),}
    login_url = reverse_lazy('member:login')
    
    def get_queryset(self):
        return BaykeShopUserAddress.objects.filter(user=self.request.user)
    

class BaykeShopUserAddressCreateView(LoginRequiredMixin, SuccessMessageMixin, RedirectURLMixin, CreateView):
    """ 新增收货地址 """
    template_name = 'baykeshop/member/address_form.html'
    model = BaykeShopUserAddress
    form_class = BaykeShopUserAddressForm
    extra_context = {'title': _('新增收货地址'),}
    login_url = reverse_lazy('member:login')
    success_url = reverse_lazy('member:address-list')
    success_message = _('新增收货地址成功')
    redirect_field_name = 'next'

    def get_queryset(self):
        return BaykeShopUserAddress.objects.filter(user=self.request.user)
    
    def get_redirect_url(self):
        return self.request.GET.get(self.redirect_field_name) or self.success_url

    def form_valid(self, form):
        form.instance.user = self.request.user
        if form.cleaned_data['is_default']:
            BaykeShopUserAddress.objects.filter(user=self.request.user, is_default=True).update(is_default=False)
        return super().form_valid(form)
    

class BaykeShopUserAddressUpdateView(LoginRequiredMixin, SuccessMessageMixin, RedirectURLMixin, UpdateView):
    """ 修改收货地址 """
    template_name = 'baykeshop/member/address_form.html'
    model = BaykeShopUserAddress
    form_class = BaykeShopUserAddressForm
    extra_context = {'title': _('修改收货地址'),}
    login_url = reverse_lazy('member:login')
    success_url = reverse_lazy('member:address-list')
    success_message = _('修改收货地址成功')
    redirect_field_name = 'next'

    def get_redirect_url(self):
        return self.request.GET.get(self.redirect_field_name) or self.success_url
    
    def get_queryset(self):
        return BaykeShopUserAddress.objects.filter(user=self.request.user)
    
    def form_valid(self, form):
        form.instance.user = self.request.user
        if form.cleaned_data['is_default']:
            BaykeShopUserAddress.objects.filter(user=self.request.user, is_default=True).update(is_default=False)
        return super().form_valid(form)
    

class BaykeShopUserAddressDeleteView(LoginRequiredMixin, SuccessMessageMixin, DeleteView):
    """ 删除收货地址 """
    model = BaykeShopUserAddress
    success_url = reverse_lazy('member:address-list')
    success_message = _('删除收货地址成功')
    login_url = reverse_lazy('member:login')
    
    def get_queryset(self):
        return BaykeShopUserAddress.objects.filter(user=self.request.user)


class BaykeShopUserProfileUpdateView(LoginRequiredMixin, SuccessMessageMixin, UpdateView):
    """ 修改个人资料 """
    template_name = 'baykeshop/member/profile_form.html'
    model = BaykeShopUser
    form_class = BaykeShopUserProfileForm
    extra_context ={'title': _('修改个人资料'),}
    login_url = reverse_lazy('member:login')
    success_url = reverse_lazy('member:profile')
    success_message = _('修改个人资料成功')

    def get_object(self):
        instance, iscreated = BaykeShopUser.objects.get_or_create(
            user=self.request.user, defaults={
                'user': self.request.user,
                'nickname': self.request.user.username
            })
        return instance
    
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['initial']['email'] = self.request.user.email
        return kwargs

    def form_valid(self, form):
        form.instance.user = self.request.user
        if form.cleaned_data['email']:
            user = self.request.user
            user.email = form.cleaned_data['email']
            user.save()
        return super().form_valid(form)



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from .base import BaseModel
from .fields import RichTextField

User = get_user_model()


class BaseGoodsModel(BaseModel):
    """商品表"""

    class Status(models.IntegerChoices):
        """商品状态"""

        ONLINE = 1, _("上架")
        OFFLINE = 2, _("下架")

    # 商品类型
    class GoodsType(models.IntegerChoices):
        """商品类型"""

        NORMAL = 1, _("普通商品")
        VIRTUAL = 2, _("虚拟商品")

    name = models.CharField(max_length=50, verbose_name=_("商品名称"))
    keywords = models.CharField(
        max_length=255, blank=True, null=True, verbose_name=_("商品关键字")
    )
    description = models.TextField(blank=True, null=True, verbose_name=_("商品描述"))
    # detail = models.TextField(blank=True, null=True, verbose_name=_("商品详情"))
    detail = RichTextField(blank=True, null=True, verbose_name=_("商品详情"))
    status = models.PositiveSmallIntegerField(
        choices=Status.choices, default=Status.ONLINE, verbose_name=_("商品状态")
    )
    goods_type = models.PositiveSmallIntegerField(
        choices=GoodsType.choices, default=GoodsType.NORMAL, verbose_name=_("商品类型")
    )

    class Meta:
        verbose_name = _("商品")
        verbose_name_plural = _("商品")
        ordering = ["-created_time"]
        abstract = True


class BaseGoodsSKUModel(BaseModel):
    """商品规格"""

    specs = models.JSONField(
        verbose_name=_("规格"), default=list, help_text=_("规格数据"), blank=True
    )
    price = models.DecimalField(max_digits=10, decimal_places=2, verbose_name=_("价格"))
    sku_sn = models.CharField(
        max_length=50, verbose_name=_("商品编码"), blank=True, default=""
    )
    # 划线价
    line_price = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        verbose_name=_("划线价"),
        blank=True,
        default="",
        help_text=_("商品价格划线价，不参与价位计算筛选, 仅供前端显示使用"),
    )
    stock = models.PositiveSmallIntegerField(default=0, verbose_name=_("库存"))
    sales = models.PositiveSmallIntegerField(default=0, verbose_name=_("销量"))

    class Meta:
        verbose_name = _("商品规格")
        verbose_name_plural = _("商品规格")
        ordering = ["-created_time"]
        abstract = True



from django.forms import widgets
from django.utils.translation import gettext_lazy as _


class Input(widgets.Input):

    template_name = 'baykeshop/forms/widgets/input.html'

    def __init__(self, 
                attrs=None,
                icon_position=None, 
                icons_class=None):
        super().__init__(attrs)
        # 图标位置bk-has-icons-left bk-has-icons-right
        self.icon_position = icon_position or ''
        self.icons_class = icons_class or { 'left': '', 'right': '' }
        
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['icon_position'] = self.icon_position
        context['icons_class'] = self.icons_class
        return context
    
    def build_attrs(self, base_attrs, extra_attrs=None):
        extra_attrs.update({
            'class': 'bk-input'
        })
        return super().build_attrs(base_attrs, extra_attrs)


class TextInput(Input):
    """ 文本输入框 """
    input_type = 'text'
    template_name = 'baykeshop/forms/widgets/text.html'
    

class PasswordInput(Input):
    """ 密码输入框 """

    input_type = 'password'
    template_name = 'baykeshop/forms/widgets/password.html'

    def __init__(self, attrs=None, icon_position=None, icons_class=None, render_value=False):
        super().__init__(attrs, icon_position, icons_class)
        self.render_value = render_value
    
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['render_value'] = self.render_value
        return context


class Select(widgets.Select):
    """ 下拉框 """
    template_name = 'baykeshop/forms/widgets/select.html'
    
    def __init__(
            self, attrs=None, choices=(), icon_position=None, icons_class=None, select_class=None
        ):
        super().__init__(attrs, choices)
        self.icon_position = icon_position or ''
        # 仅支持左侧配置
        self.icons_class = icons_class or { 'left': '', 'right': '' }
        # select样式,可以影响颜色，大小，状态
        # bk-is-multiple 支持多选
        # bk-is-rounded 圆角
        # bk-is-link等颜色标签，会影响颜色
        # bk-is-small bk-is-medium bk-is-large 大小
        # bk-is-loading 加载中
        # bk-is-hovered 鼠标移上去
        # bk-is-focused 获取焦点
        # bk-is-active 选中
        self.select_class = select_class or ''
    
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['icon_position'] = self.icon_position
        context['icons_class'] = self.icons_class
        return context
    

class RichTextWidget(widgets.Textarea):
    """ 富文本 """
    template_name = 'baykeshop/forms/widgets/richtext.html'

    class Media:
        js = ('baykeshop/tinymce/tinymce.min.js',)


from django.db import models

from baykeshop.db.base import BaseManager


class BaykeShopGoodsManager(BaseManager):
    """商品管理器"""
    def get_queryset(self):
        from baykeshop.contrib.shop.models import BaykeShopGoodsImages
        return super().get_queryset().alias(
            price=models.Min('baykeshopgoodssku__price'),
            sales=models.Sum('baykeshopgoodssku__sales'),
            stock=models.Min('baykeshopgoodssku__stock'),
            line_price=models.Min('baykeshopgoodssku__line_price'),
        ).annotate(
            price=models.F('price'),
            sales=models.F('sales'),
            stock=models.F('stock'),
            line_price=models.F('line_price'),
            image_url = models.Subquery(
                BaykeShopGoodsImages.objects.filter(
                    goods=models.OuterRef('pk'),
                ).values('image')[:1],
                output_field=models.CharField()
            )
        ).prefetch_related('baykeshopgoodssku_set', 'baykeshopgoodsimages_set')


class BaykeShopCartsManager(BaseManager):
    """购物车管理器"""
    def get_queryset(self):
        from baykeshop.contrib.shop.models import BaykeShopGoodsImages
        return super().get_queryset().select_related('sku').alias(
            total_price=models.ExpressionWrapper(
                models.F('quantity') * models.F('sku__price'),
                output_field=models.DecimalField()
            ),
            name=models.F('sku__goods__name'),
            specs=models.F('sku__specs'),
            price=models.F('sku__price'),
        ).annotate(
            total_price=models.F('total_price'),
            name=models.F('name'),
            specs=models.F('specs'),
            image_url = models.Subquery(
                BaykeShopGoodsImages.objects.filter(
                    goods=models.OuterRef('sku__goods'),
                ).values('image')[:1],
                output_field=models.CharField()
            ),
            price=models.F('price'),
        )
    

class BaykeShopGoodsSKUManager(models.Manager):
    """ sku 管理器 """
    def get_queryset(self):
        from baykeshop.contrib.shop.models import BaykeShopGoodsImages
        queryset = super().get_queryset().select_related('goods').alias(
            name=models.F('goods__name'),
            detail=models.F('goods__detail'),
            goods_type=models.F('goods__goods_type'),
            image_url=models.Subquery(
                BaykeShopGoodsImages.objects.filter(
                    goods=models.OuterRef('goods'),
                ).values('image')[:1],
                output_field=models.CharField()
            )
        ).annotate(
            name = models.F('goods__name'),
            image_url = models.F('image_url'),
            detail=models.F('goods__detail'),
            goods_type=models.F('goods__goods_type'),
        )
        return queryset


class BaykeShopOrdersGoodsManager(BaseManager):
    """订单商品管理器"""
    def get_queryset(self):
        from baykeshop.contrib.shop.models import BaykeShopGoodsImages
        return super().get_queryset().select_related('sku').alias(
            name=models.F('sku__goods__name'),
            specs=models.F('sku__specs'),
            image_url = models.Subquery(
                BaykeShopGoodsImages.objects.filter(
                    goods=models.OuterRef('sku__goods'),
                ).values('image')[:1],
                output_field=models.CharField()
            )
        )


from django.forms import fields
from baykeshop.forms import widgets


class RichTextField(fields.CharField):
    """ 富文本编辑器 """

    widget = widgets.RichTextWidget()
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = kwargs.get('widget', self.widget)
        super().__init__(*args, **kwargs)


from django.urls import path, include


urlpatterns = [
    path('member/', include('baykeshop.contrib.member.urls')),
    path('article/', include('baykeshop.contrib.article.urls')),
    path('', include('baykeshop.contrib.shop.urls')),
    # 接口url
    path('api/', include('baykeshop.api.urls', namespace='baykeshop_api')),
]


from rest_framework import mixins, viewsets
from rest_framework import authentication
from rest_framework import permissions

from baykeshop.contrib.shop.models import BaykeShopOrders
from .serializers import BaykeShopOrdersPaySerializer


class BaykeShopOrdersPayView(mixins.UpdateModelMixin, viewsets.GenericViewSet):
    """
    支付订单
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    queryset = BaykeShopOrders.objects.all()
    serializer_class = BaykeShopOrdersPaySerializer
    lookup_field = 'order_sn'
    lookup_url_kwarg = 'order_sn'
    
    def get_queryset(self):
        return super().get_queryset().filter(user=self.request.user)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :__init__.py
@说明    :封装支付宝支付
@时间    :2024/12/09 22:09:51
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''


from alipay.aop.api.AlipayClientConfig import AlipayClientConfig
from alipay.aop.api.DefaultAlipayClient import DefaultAlipayClient
from alipay.aop.api.domain.AlipayTradePagePayModel import AlipayTradePagePayModel
from alipay.aop.api.request.AlipayTradePagePayRequest import AlipayTradePagePayRequest

from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.contrib.shop.models import BaykeShopOrders


class AliPay:

    def __init__(
        self,
        appid=None,
        app_private_key=None,
        alipay_public_key=None,
        return_url=None,
        notify_url=None,
        debug=False,
        **kwargs,
    ):
        self.app_id = appid or BaykeDictModel.get_key_value("ALIPAY_APPID")
        self.app_private_key = app_private_key or BaykeDictModel.get_key_value(
            "ALIPAY_PRIVATE_KEY"
        )
        self.alipay_public_key = alipay_public_key or BaykeDictModel.get_key_value(
            "ALIPAY_PUBLIC_KEY"
        )
        self.return_url = return_url
        self.notify_url = notify_url
        self.debug = debug
        self.extra_params = kwargs

    @property
    def server_url(self):
        if self.debug:
            return "https://openapi-sandbox.dl.alipaydev.com/gateway.do"
        return "https://openapi.alipay.com/gateway.do"

    @property
    def alipay_config(self):
        """支付宝配置"""
        config = AlipayClientConfig(sandbox_debug=self.debug)
        config.app_id = self.app_id
        config.app_private_key = self.app_private_key
        config.alipay_public_key = self.alipay_public_key
        config.server_url = self.server_url
        config.sign_type = self.extra_params.get("sign_type", "RSA2")
        return config

    def alipay_client(self):
        """支付宝客户端"""
        logger = self.extra_params.get("logger")
        return DefaultAlipayClient(
            alipay_client_config=self.alipay_config, logger=logger
        )

    def trade_page_pay_model(self, instance:BaykeShopOrders):
        """ 支付宝网页支付商品信息
        :param instance: 订单实例
        :return: AlipayTradePagePayModel
        """
        model = AlipayTradePagePayModel()
        model.out_trade_no = instance.order_sn
        model.total_amount = str(instance.pay_price)
        model.subject = instance.order_sn
        model.product_code = "FAST_INSTANT_TRADE_PAY"
        return model
    
    
    def trade_page_pay_request(self, instance):
        """ 支付宝支付请求
        :param instance: BaykeShopOrders
        :return: AlipayTradePagePayRequest
        """
        request = AlipayTradePagePayRequest()
        request.biz_model = self.trade_page_pay_model(instance)
        request.return_url = self.return_url
        request.notify_url = self.notify_url
        return request
    
    def trade_page_pay(self, instance):
        """ 创建支付宝支付请求
        :param instance:BaykeShopOrders
        :return: 支付宝支付链接
        """
        client = self.alipay_client()
        request = self.trade_page_pay_request(instance)
        res = client.page_execute(request, http_method="GET")
        return res
